@@ -4,9 +4,15 @@ use regex::Regex;
 use std::env;
 use std::path::PathBuf;
 
+const D3D12_AGILITY_SDK_INCLUDE_PATH: &str =
+    "vendored\\D3D12AgilitySDK\\include";
+const D3D12_AGILITY_SDK_LIB_PATH: &str = "vendored\\D3D12AgilitySDK\\bin";
+
+const PIX_INCLUDE_PATH: &str = "vendored\\WinPixEventRuntime\\include";
+const PIX_LIB_PATH: &str = "vendored\\WinPixEventRuntime\\bin";
+
 fn find_d3d12_header() -> Option<String> {
-    let path = std::path::Path::new("D3D12AgilitySDK")
-        .join("Include")
+    let path = PathBuf::from(D3D12_AGILITY_SDK_INCLUDE_PATH)
         .join("d3d12.h")
         .to_str()
         .expect("Path to Agility SDK is not valid UTF-8")
@@ -90,26 +96,72 @@ fn patch_d3d12_header() -> String {
 }
 
 fn main() {
-    // Tell cargo to tell rustc to link the system d3d12 and dxgi
-    // shared libraries.
+    let workspace_dir = PathBuf::from(env::var("CARGO_MANIFEST_DIR").unwrap());
+
+    println!("cargo:rustc-link-search={}", D3D12_AGILITY_SDK_LIB_PATH);
     println!("cargo:rustc-link-lib=d3d12");
     println!("cargo:rustc-link-lib=dxgi");
     // we have no __uuidof, so let's use oldschool way
     println!("cargo:rustc-link-lib=dxguid");
+    println!(
+        "cargo:rustc-link-arg=/DEF:{}\\agility.def",
+        D3D12_AGILITY_SDK_LIB_PATH
+    );
+
+    #[cfg(feature = "devel")]
+    generate_bindings();
+
+    // Our PIX wrapper has an extra layer - C wrapper around the original C++ interface
+    // Since without `devel` feature we cannot run bindgen and build it, we have to ship
+    // the pre-built library
+    println!("cargo:rustc-link-search={}", PIX_LIB_PATH);
+    println!("cargo:rustc-link-lib=static=pix_wrapper");
+    println!("cargo:rustc-link-lib=WinPixEventRuntime");
+
+    // Copy DX12 Agility SDK libs that are needed by examples
+    let copy_source_path = workspace_dir.join(D3D12_AGILITY_SDK_LIB_PATH);
+    let profile = env::var("PROFILE").unwrap();
+    let examples_bin_path =
+        workspace_dir.join("target").join(profile).join("examples");
+    let copy_dest_path = examples_bin_path.join("D3D12");
+    std::fs::create_dir_all(&copy_dest_path)
+        .expect("Cannot create D3D12 dir to copy Agility SDK dlls");
+
+    let files_to_copy = [
+        "D3D12Core.dll",
+        "D3D12Core.pdb",
+        "d3d12SDKLayers.dll",
+        "d3d12SDKLayers.pdb",
+    ];
+
+    for file in files_to_copy {
+        std::fs::copy(copy_source_path.join(file), copy_dest_path.join(file))
+            .expect("Cannot copy Agility SDK dlls");
+    }
+
+    // Copy PIX runtime DLL since it's needed by examples
+    let pix_dll_name = "WinPixEventRuntime.dll";
+    std::fs::copy(
+        &format!("{}\\{}", PIX_LIB_PATH, pix_dll_name),
+        examples_bin_path.join(pix_dll_name),
+    )
+    .expect("Cannot copy WinPixEventRuntime.dll");
+}
+
+#[cfg(feature = "devel")]
+fn generate_bindings() {
     // Tell cargo to invalidate the built crate whenever the wrapper changes
-    println!("cargo:rerun-if-changed=wrapper.h");
+    println!("cargo:rerun-if-changed=generation\\d3d12_wrapper.h");
 
-    println!("cargo:rustc-link-arg=/DEF:agility.def");
-
-    // The bindgen::Builder is the main entry point
-    // to bindgen, and lets you build up options for
-    // the resulting bindings.
+    let workspace_dir = PathBuf::from(env::var("CARGO_MANIFEST_DIR").unwrap())
+        .to_str()
+        .expect("Workspace path is not valid UTF-8")
+        .to_owned();
     let bindings = bindgen::Builder::default()
-        // The input header we would like to generate
-        // bindings for.
         .clang_arg("-std=c99")
+        // .clang_arg(&format!("-Igeneration"))
         .header_contents("d3d12_patched.h", &patch_d3d12_header())
-        .header("wrapper.h")
+        .header("generation\\d3d12_wrapper.h")
         .layout_tests(false)
         .derive_debug(true)
         .impl_debug(true)
@@ -129,74 +181,41 @@ fn main() {
         .whitelist_function("CreateEventW")
         .whitelist_function("WaitForSingleObject")
         .whitelist_function("CloseHandle")
-        // Tell cargo to invalidate the built crate whenever any of the
-        // included header files changed.
         .parse_callbacks(Box::new(bindgen::CargoCallbacks))
         // Finish the builder and generate the bindings.
         .generate()
-        // Unwrap the Result and panic on failure.
         .expect("Unable to generate bindings");
 
-    // Write the bindings to the $OUT_DIR/bindings.rs file.
     let out_path = PathBuf::from(env::var("OUT_DIR").unwrap());
     bindings
-        .write_to_file(out_path.join("bindings.rs"))
-        .expect("Couldn't write bindings!");
+        .write_to_file(out_path.join("d3d12_bindings.rs"))
+        .expect("Cannot write bindings!");
 
-    // Copy DX12 Agility SDK libs that are needed by examples
-    let manifest_path = PathBuf::from(env::var("CARGO_MANIFEST_DIR").unwrap());
-    let copy_source_path = manifest_path.join("D3D12AgilitySDK").join("bin");
-    let profile = env::var("PROFILE").unwrap();
-    let examples_bin_path =
-        manifest_path.join("target").join(profile).join("examples");
-    let copy_dest_path = examples_bin_path.join("D3D12");
-    std::fs::create_dir_all(&copy_dest_path)
-        .expect("Cannot create D3D12 dir to copy Agility SDK dlls");
-
-    let files_to_copy = [
-        "D3D12Core.dll",
-        "D3D12Core.pdb",
-        "d3d12SDKLayers.dll",
-        "d3d12SDKLayers.pdb",
-    ];
-
-    for file in files_to_copy {
-        std::fs::copy(copy_source_path.join(file), copy_dest_path.join(file))
-            .expect("Cannot copy Agility SDK dlls");
-    }
-
-    #[cfg(feature = "pix")]
-    setup_pix_wrapper();
+    generate_pix_bindings();
 }
 
-fn setup_pix_wrapper() {
-    let pix_runtime_path = PathBuf::from(env::var("PIX_RUNTIME_PATH").unwrap())
-        .to_str()
-        .unwrap()
-        .to_owned();
+#[cfg(feature = "devel")]
+fn generate_pix_bindings() {
+    let workspace_dir = PathBuf::from(env::var("CARGO_MANIFEST_DIR").unwrap());
 
     // Build C wrapper over C++ PIX header
     cc::Build::new()
         .cpp(true)
-        .include(format!("{}\\include\\", pix_runtime_path))
-        .include(
-            PathBuf::from(env::var("CARGO_MANIFEST_DIR").unwrap())
-                .join("D3D12AgilitySDK")
-                .join("include"),
-        )
-        .file("pix_wrapper.cpp")
+        .include(PIX_INCLUDE_PATH)
+        .include(workspace_dir.join(D3D12_AGILITY_SDK_INCLUDE_PATH))
+        .include(workspace_dir.join("generation"))
+        .file("generation\\pix_wrapper.cpp")
         .compile("pix_wrapper");
 
     // Generate Rust bindings to C wrapper
-    println!("cargo:rustc-link-search={}\\bin\\x64", pix_runtime_path);
-    println!("cargo:rustc-link-lib=WinPixEventRuntime");
-    println!("cargo:rerun-if-changed=pix_wrapper.h");
-    println!("cargo:rerun-if-changed=pix_wrapper.cpp");
-    println!("cargo:rustc-link-lib=static=pix_wrapper");
+    println!("cargo:rerun-if-changed=generation\\pix_wrapper.h");
+    println!("cargo:rerun-if-changed=generation\\pix_wrapper.cpp");
 
     let bindings = bindgen::Builder::default()
         .layout_tests(false)
-        .header("pix_wrapper.h")
+        .clang_arg(&format!("-I{}", D3D12_AGILITY_SDK_INCLUDE_PATH))
+        .clang_arg(&format!("-I{}", PIX_INCLUDE_PATH))
+        .header("generation\\pix_wrapper.h")
         .whitelist_function("pix_.*")
         .whitelist_type("ID3D12GraphicsCommandList.*")
         .whitelist_type("ID3D12CommandQueue.*")
