@@ -70,10 +70,11 @@ struct Vertex {
 }
 
 impl Vertex {
-    fn make_desc() -> InputLayout {
+    fn make_desc() -> Vec<InputElementDesc<'static>> {
         vec![InputElementDesc::default()
-            .set_name(CString::new("POSITION").unwrap())
-            .set_format(DxgiFormat::R32G32B32_Float)
+            .set_name("POSITION")
+            .unwrap()
+            .set_format(Format::R32G32B32_Float)
             .set_input_slot(0)
             .set_offset(Bytes(offset_of!(Self, position) as u64))]
     }
@@ -112,7 +113,7 @@ struct HelloMeshShadersSample {
     fence: Fence,
     fence_event: Win32Event,
     fence_values: [u64; FRAMES_IN_FLIGHT as usize],
-    swapchain: DxgiSwapchain,
+    swapchain: Swapchain,
     frame_index: usize,
     frame_count: u32,
     viewport_desc: Viewport,
@@ -127,7 +128,7 @@ struct HelloMeshShadersSample {
 
     depth_stencil: Option<Resource>,
 
-    meshlet_count: Elements,
+    meshlet_count: u32,
     vertex_buffer: Option<Resource>,
     meshlet_buffer: Option<Resource>,
     triangle_indices_buffer: Option<Resource>,
@@ -141,16 +142,16 @@ impl HelloMeshShadersSample {
     fn new(hwnd: *mut std::ffi::c_void, break_on_warn: bool) -> Self {
         trace!("Creating app instance");
 
-        let mut factory_flags = DxgiCreateFactoryFlags::None;
+        let mut factory_flags = CreateFactoryFlags::None;
         if USE_DEBUG {
             let debug_controller =
                 Debug::new().expect("Cannot create debug controller");
             debug_controller.enable_debug_layer();
-            factory_flags = DxgiCreateFactoryFlags::Debug;
+            factory_flags = CreateFactoryFlags::Debug;
         }
 
         let factory =
-            DxgiFactory::new(factory_flags).expect("Cannot create factory");
+            Factory::new(factory_flags).expect("Cannot create factory");
 
         let device = create_device(&factory);
 
@@ -261,7 +262,7 @@ impl HelloMeshShadersSample {
             command_list,
             depth_stencil: None,
 
-            meshlet_count: 0.into(),
+            meshlet_count: 0,
             vertex_buffer: None,
             meshlet_buffer: None,
             triangle_indices_buffer: None,
@@ -303,7 +304,7 @@ impl HelloMeshShadersSample {
         self.vertex_buffer =
             Some(self.create_default_buffer(&vertices, "VertexBuffer"));
 
-        self.meshlet_count = Elements::from(meshlets.len());
+        self.meshlet_count = meshlets.len() as u32;
         self.meshlet_buffer =
             Some(self.create_default_buffer(&meshlets, "MeshletBuffer"));
         self.triangle_indices_buffer =
@@ -332,11 +333,11 @@ impl HelloMeshShadersSample {
         let staging_buffer = self
             .device
             .create_committed_resource(
-                &HeapProperties::default().set_type(HeapType::Upload),
+                &HeapProperties::default().set_heap_type(HeapType::Upload),
                 HeapFlags::None,
                 &ResourceDesc::default()
                     .set_dimension(ResourceDimension::Buffer)
-                    .set_width(Elements(size.0))
+                    .set_width(size.0)
                     .set_layout(TextureLayout::RowMajor),
                 ResourceStates::GenericRead,
                 None,
@@ -348,7 +349,7 @@ impl HelloMeshShadersSample {
             .expect("Cannot set name on staging buffer");
 
         let data = staging_buffer
-            .map(Elements(0), None)
+            .map(0, None)
             .expect("Cannot map staging buffer");
 
         unsafe {
@@ -363,11 +364,11 @@ impl HelloMeshShadersSample {
         let default_buffer = self
             .device
             .create_committed_resource(
-                &HeapProperties::default().set_type(HeapType::Default),
+                &HeapProperties::default().set_heap_type(HeapType::Default),
                 HeapFlags::None,
                 &ResourceDesc::default()
                     .set_dimension(ResourceDimension::Buffer)
-                    .set_width(Elements(size.0))
+                    .set_width(size.0)
                     .set_layout(TextureLayout::RowMajor),
                 ResourceStates::CopyDest,
                 None,
@@ -379,7 +380,7 @@ impl HelloMeshShadersSample {
             .expect("Cannot set name on default buffer");
 
         // self.command_list
-        //     .resource_barrier(&[ResourceBarrier::transition(
+        //     .resource_barrier(&[ResourceBarrier::new_transition(
         //         &ResourceTransitionBarrier::default()
         //             .set_resource(&default_buffer)
         //             .set_state_before(ResourceStates::CommonOrPresent)
@@ -395,7 +396,7 @@ impl HelloMeshShadersSample {
         );
 
         self.command_list
-            .resource_barrier(&[ResourceBarrier::transition(
+            .resource_barrier(&[ResourceBarrier::new_transition(
                 &ResourceTransitionBarrier::default()
                     .set_resource(&default_buffer)
                     .set_state_before(ResourceStates::CopyDest)
@@ -426,35 +427,35 @@ impl HelloMeshShadersSample {
             .set_graphics_root_signature(&self.root_signature);
 
         self.command_list.set_graphics_root_constant_buffer_view(
-            Elements(0),
+            0,
             self.constant_buffer
                 .as_ref()
                 .expect("No constant buffer created")
                 .get_gpu_virtual_address(),
         );
         self.command_list.set_graphics_root_shader_resource_view(
-            Elements(1),
+            1,
             self.vertex_buffer
                 .as_ref()
                 .expect("No vertex buffer created")
                 .get_gpu_virtual_address(),
         );
         self.command_list.set_graphics_root_shader_resource_view(
-            Elements(2),
+            2,
             self.meshlet_buffer
                 .as_ref()
                 .expect("No meshlet buffer created")
                 .get_gpu_virtual_address(),
         );
         self.command_list.set_graphics_root_shader_resource_view(
-            Elements(3),
+            3,
             self.triangle_indices_buffer
                 .as_ref()
                 .expect("No triangle index buffer created")
                 .get_gpu_virtual_address(),
         );
         self.command_list.set_graphics_root_shader_resource_view(
-            Elements(4),
+            4,
             self.vertex_indices_buffer
                 .as_ref()
                 .expect("No vertex index buffer created")
@@ -465,22 +466,21 @@ impl HelloMeshShadersSample {
         self.command_list
             .set_scissor_rects(&vec![self.scissor_desc]);
 
-        self.command_list
-            .resource_barrier(&vec![ResourceBarrier::transition(
+        self.command_list.resource_barrier(&vec![
+            ResourceBarrier::new_transition(
                 &ResourceTransitionBarrier::default()
                     .set_resource(
                         &self.render_targets[self.frame_index as usize],
                     )
                     .set_state_before(ResourceStates::CommonOrPresent)
                     .set_state_after(ResourceStates::RenderTarget),
-            )]);
+            ),
+        ]);
 
         let rtv_handle = self
             .rtv_heap
             .get_cpu_descriptor_handle_for_heap_start()
-            .advance(Elements(
-                self.swapchain.get_current_back_buffer_index().0 as u64
-            ));
+            .advance(self.swapchain.get_current_back_buffer_index());
 
         self.command_list.set_render_targets(
             &mut [rtv_handle],
@@ -497,14 +497,10 @@ impl HelloMeshShadersSample {
 
         // ToDo: DSV
 
-        self.command_list.dispatch_mesh(
-            self.meshlet_count,
-            Elements(1),
-            Elements(1),
-        );
+        self.command_list.dispatch_mesh(self.meshlet_count, 1, 1);
 
         self.command_list
-            .resource_barrier(&[ResourceBarrier::transition(
+            .resource_barrier(&[ResourceBarrier::new_transition(
                 &ResourceTransitionBarrier::default()
                     .set_resource(
                         &self.render_targets[self.frame_index as usize],
@@ -533,7 +529,7 @@ impl HelloMeshShadersSample {
                 .set_event_on_completion(last_fence_value, &self.fence_event)
                 .expect("Cannot set event on fence");
 
-            self.fence_event.wait();
+            self.fence_event.wait(None);
         }
 
         self.populate_command_list(self.frame_index);
@@ -541,7 +537,9 @@ impl HelloMeshShadersSample {
         self.command_queue
             .execute_command_lists(std::slice::from_ref(&self.command_list));
 
-        self.swapchain.present(1, 0).expect("Cannot present");
+        self.swapchain
+            .present(1, PresentFlags::None)
+            .expect("Cannot present");
 
         self.fence_values[self.frame_index] = last_fence_value + 1;
         trace!(
@@ -576,7 +574,7 @@ impl HelloMeshShadersSample {
             self.fence
                 .set_event_on_completion(fence_value, &self.fence_event)
                 .expect("Cannot set event on fence");
-            self.fence_event.wait();
+            self.fence_event.wait(None);
         }
     }
 
@@ -584,20 +582,20 @@ impl HelloMeshShadersSample {
         let _debug_printer = make_debug_printer!(&self.info_queue);
 
         let depth_stencil_desc = DepthStencilViewDesc::default()
-            .set_format(DxgiFormat::D32_Float)
+            .set_format(Format::D32_Float)
             .set_view_dimension(DsvDimension::Texture2D)
             .set_flags(DsvFlags::None);
 
         let depth_stencil = self
             .device
             .create_committed_resource(
-                &HeapProperties::default().set_type(HeapType::Default),
+                &HeapProperties::default().set_heap_type(HeapType::Default),
                 HeapFlags::None,
                 &ResourceDesc::default()
                     .set_dimension(ResourceDimension::Texture2D)
                     .set_width(WINDOW_WIDTH.into())
                     .set_height(WINDOW_HEIGHT.into())
-                    .set_format(DxgiFormat::D32_Float)
+                    .set_format(Format::D32_Float)
                     .set_flags(
                         ResourceFlags::AllowDepthStencil
                             | ResourceFlags::DenyShaderResource,
@@ -605,7 +603,7 @@ impl HelloMeshShadersSample {
                 ResourceStates::DepthWrite,
                 Some(
                     &ClearValue::default()
-                        .set_format(DxgiFormat::D32_Float)
+                        .set_format(Format::D32_Float)
                         .set_depth_stencil(
                             &DepthStencilValue::default()
                                 .set_depth(1.)
@@ -632,11 +630,11 @@ impl HelloMeshShadersSample {
         let constant_buffer = self
             .device
             .create_committed_resource(
-                &HeapProperties::default().set_type(HeapType::Upload),
+                &HeapProperties::default().set_heap_type(HeapType::Upload),
                 HeapFlags::None,
                 &ResourceDesc::default()
                     .set_dimension(ResourceDimension::Buffer)
-                    .set_width((size_of::<MeshletConstantBuffer>()).into())
+                    .set_width(size_of::<MeshletConstantBuffer>() as u64)
                     .set_layout(TextureLayout::RowMajor),
                 ResourceStates::GenericRead,
                 None,
@@ -644,7 +642,7 @@ impl HelloMeshShadersSample {
             .expect("Cannot create cbuffer staging buffer");
 
         let constant_buffer_ptr = constant_buffer
-            .map(Elements(0), None)
+            .map(0, None)
             .expect("Cannot map cbv staging buffer");
 
         let camera = Camera::default();
@@ -691,14 +689,14 @@ fn create_pipeline_state(
     let pso_subobjects_desc = MeshShaderPipelineStateDesc::default()
         .set_root_signature(root_signature)
         .set_mesh_shader_bytecode(&ms_bytecode)
-        .set_pixel_shader_bytecode(&ps_bytecode)
+        .set_ps_bytecode(&ps_bytecode)
         .set_rasterizer_state(
             &RasterizerDesc::default().set_cull_mode(CullMode::Back),
         )
         .set_blend_state(&BlendDesc::default())
         .set_depth_stencil_state(&DepthStencilDesc::default())
         .set_primitive_topology_type(PrimitiveTopologyType::Triangle)
-        .set_rtv_formats(&[DxgiFormat::R8G8B8A8_UNorm]);
+        .set_rtv_formats(&[Format::R8G8B8A8_UNorm]);
 
     let pso_desc = PipelineStateStreamDesc::default()
         .set_pipeline_state_subobject_stream(
@@ -751,13 +749,13 @@ fn setup_root_signature(
 
 fn setup_heaps(
     device: &Device,
-    swapchain: &DxgiSwapchain,
+    swapchain: &Swapchain,
 ) -> (Vec<Resource>, DescriptorHeap, DescriptorHeap) {
     let rtv_heap = device
         .create_descriptor_heap(
             &DescriptorHeapDesc::default()
-                .set_type(DescriptorHeapType::RTV)
-                .set_num_descriptors(Elements::from(FRAMES_IN_FLIGHT)),
+                .set_heap_type(DescriptorHeapType::RTV)
+                .set_num_descriptors(u32::from(FRAMES_IN_FLIGHT)),
         )
         .expect("Cannot create RTV heap");
     rtv_heap
@@ -767,8 +765,8 @@ fn setup_heaps(
     let dsv_heap = device
         .create_descriptor_heap(
             &DescriptorHeapDesc::default()
-                .set_type(DescriptorHeapType::DSV)
-                .set_num_descriptors(Elements(1)),
+                .set_heap_type(DescriptorHeapType::DSV)
+                .set_num_descriptors(1),
         )
         .expect("Cannot create RTV heap");
     dsv_heap
@@ -780,40 +778,37 @@ fn setup_heaps(
     let mut render_targets = vec![];
     for frame_idx in 0..FRAMES_IN_FLIGHT {
         let render_target = swapchain
-            .get_buffer(Elements::from(frame_idx))
+            .get_buffer(u32::from(frame_idx))
             .expect("cannot get buffer from swapchain");
 
         device.create_render_target_view(&render_target, rtv_handle);
         render_targets.push(render_target);
 
-        rtv_handle = rtv_handle.advance(Elements(1));
+        rtv_handle = rtv_handle.advance(1);
     }
 
     (render_targets, rtv_heap, dsv_heap)
 }
 
 fn create_swapchain(
-    factory: DxgiFactory,
+    factory: Factory,
     command_queue: &CommandQueue,
     hwnd: *mut std::ffi::c_void,
-) -> DxgiSwapchain {
-    let swapchain_desc = DxgiSwapchainDesc::default()
+) -> Swapchain {
+    let swapchain_desc = SwapchainDesc::default()
         .set_width(WINDOW_WIDTH)
         .set_height(WINDOW_HEIGHT)
-        .set_buffer_count(Elements::from(FRAMES_IN_FLIGHT));
+        .set_buffer_count(u32::from(FRAMES_IN_FLIGHT));
     let swapchain = factory
         .create_swapchain(&command_queue, hwnd as *mut HWND__, &swapchain_desc)
         .expect("Cannot create swapchain");
     factory
-        .make_window_association(
-            hwnd,
-            DxgiMakeWindowAssociationFlags::NoAltEnter,
-        )
+        .make_window_association(hwnd, MakeWindowAssociationFlags::NoAltEnter)
         .expect("Cannot make window association");
     swapchain
 }
 
-fn create_device(factory: &DxgiFactory) -> Device {
+fn create_device(factory: &Factory) -> Device {
     let device;
     if USE_WARP_ADAPTER {
         let warp_adapter = factory
@@ -978,6 +973,9 @@ fn load_mesh() -> (Vec<Vertex>, Vec<Meshlet>, Vec<u32>, Vec<u32>) {
     (vertices, meshlets, triangle_indices, vertex_indices)
 }
 
+#[derive(Debug, Copy, Clone)]
+pub struct Radians(pub f32);
+
 #[derive(Debug, Copy, Clone)]
 pub struct Degrees(pub f32);
 
